@@ -727,6 +727,63 @@ pyg_value_array_from_pyobject(GValue *value,
     return 0;
 }
 
+static int
+pyg_array_from_pyobject(GValue *value,
+		        PyObject *obj)
+{
+    int len;
+    GArray *array;
+    int i;
+
+    len = PySequence_Length(obj);
+    if (len == -1) {
+	PyErr_Clear();
+	return -1;
+    }
+
+    array = g_array_new(FALSE, TRUE, sizeof(GValue));
+
+    for (i = 0; i < len; ++i) {
+	PyObject *item = PySequence_GetItem(obj, i);
+	GType type;
+	GValue item_value = { 0, };
+	int status;
+
+	if (! item) {
+	    PyErr_Clear();
+	    g_array_free(array, FALSE);
+	    return -1;
+	}
+
+	if (item == Py_None)
+	    type = G_TYPE_POINTER; /* store None as NULL */
+	else {
+	    type = pyg_type_from_object((PyObject*)Py_TYPE(item));
+	    if (! type) {
+		PyErr_Clear();
+		g_array_free(array, FALSE);
+		Py_DECREF(item);
+		return -1;
+	    }
+	}
+
+	g_value_init(&item_value, type);
+	status = pyg_value_from_pyobject(&item_value, item);
+	Py_DECREF(item);
+
+	if (status == -1) {
+	    g_array_free(array, FALSE);
+	    g_value_unset(&item_value);
+	    return -1;
+	}
+
+	g_array_append_val(array, item_value);
+    }
+
+    g_value_take_boxed(value, array);
+    return 0;
+}
+
 /**
  * pyg_value_from_pyobject:
  * @value: the GValue object to store the converted value in.
@@ -766,12 +823,20 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
 	}
 	break;
     case G_TYPE_CHAR:
+	if (PYGLIB_PyLong_Check(obj)) {
+	    glong val;
+	    val = PYGLIB_PyLong_AsLong(obj);
+	    if (val >= -128 && val <= 127)
+		g_value_set_schar(value, (gchar) val);
+	    else
+		return -1;
+	}
 #if PY_VERSION_HEX < 0x03000000
-	if (PyString_Check(obj)) {
+	else if (PyString_Check(obj)) {
 	    g_value_set_schar(value, PyString_AsString(obj)[0]);
-	} else
+	}
 #endif
-	if (PyUnicode_Check(obj)) {
+	else if (PyUnicode_Check(obj)) {
 	    tmp = PyUnicode_AsUTF8String(obj);
 	    g_value_set_schar(value, PYGLIB_PyBytes_AsString(tmp)[0]);
 	    Py_DECREF(tmp);
@@ -786,7 +851,7 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
 	    glong val;
 	    val = PYGLIB_PyLong_AsLong(obj);
 	    if (val >= 0 && val <= 255)
-	      g_value_set_uchar(value, (guchar)PYGLIB_PyLong_AsLong (obj));
+	      g_value_set_uchar(value, (guchar) val);
 	    else
 	      return -1;
 #if PY_VERSION_HEX < 0x03000000
@@ -959,6 +1024,9 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
         else if (PySequence_Check(obj) &&
 		   G_VALUE_HOLDS(value, G_TYPE_VALUE_ARRAY))
 	    return pyg_value_array_from_pyobject(value, obj, NULL);
+        else if (PySequence_Check(obj) &&
+		   G_VALUE_HOLDS(value, G_TYPE_ARRAY))
+	    return pyg_array_from_pyobject(value, obj);
 	else if (PYGLIB_PyUnicode_Check(obj) &&
                  G_VALUE_HOLDS(value, G_TYPE_GSTRING)) {
             GString *string;
@@ -980,7 +1048,11 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
 	break;
     }
     case G_TYPE_PARAM:
-	if (PyGParamSpec_Check(obj))
+        /* we need to support both the wrapped _gobject.GParamSpec and the GI
+         * GObject.ParamSpec */
+        if (G_IS_PARAM_SPEC (pygobject_get (obj)))
+	    g_value_set_param(value, G_PARAM_SPEC (pygobject_get (obj)));
+        else if (PyGParamSpec_Check(obj))
 	    g_value_set_param(value, PYGLIB_CPointer_GetPointer(obj, NULL));
 	else
 	    return -1;
